@@ -1,10 +1,10 @@
 
 /* pngrutil.c - utilities to read a png file
 
-   libpng 1.0 beta 2 - version 0.8
+   libpng 1.0 beta 2 - version 0.81
    For conditions of distribution and use, see copyright notice in png.h
    Copyright (c) 1995 Guy Eric Schalnat, Group 42, Inc.
-   August 20, 1995
+   August 24, 1995
    */
 
 #define PNG_INTERNAL
@@ -38,9 +38,9 @@ png_get_uint_16(png_byte *buf)
 
 /* read data, and run it through the crc */
 void
-png_crc_read(png_struct *png_ptr, png_byte *buf, png_uint_32 length)
+png_crc_read(png_struct *png_ptr, png_bytef *buf, png_uint_32 length)
 {
-   png_read_data(png_ptr, buf, length);
+   (*(png_ptr->read_data_fn))(png_ptr, buf, length);
    png_calculate_crc(png_ptr, buf, length);
 }
 
@@ -52,12 +52,12 @@ png_crc_skip(png_struct *png_ptr, png_uint_32 length)
 
    for (i = length; i > png_ptr->zbuf_size; i -= png_ptr->zbuf_size)
    {
-      png_read_data(png_ptr, png_ptr->zbuf, png_ptr->zbuf_size);
+      (*(png_ptr->read_data_fn))(png_ptr, png_ptr->zbuf, png_ptr->zbuf_size);
       png_calculate_crc(png_ptr, png_ptr->zbuf, png_ptr->zbuf_size);
    }
    if (i)
    {
-      png_read_data(png_ptr, png_ptr->zbuf, i);
+      (*(png_ptr->read_data_fn))(png_ptr, png_ptr->zbuf, i);
       png_calculate_crc(png_ptr, png_ptr->zbuf, i);
    }
 }
@@ -73,7 +73,7 @@ png_handle_IHDR(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    /* check the length */
    if (length != 13)
-      png_error(png_ptr, "Invalid IHDR chunk");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid IHDR chunk");
 
    png_crc_read(png_ptr, buf, 13);
 
@@ -87,36 +87,36 @@ png_handle_IHDR(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    /* check for width and height valid values */
    if (width == 0 || height == 0)
-      png_error(png_ptr, "Invalid Width or Height Found");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid Width or Height Found");
 
    /* check other values */
    if (bit_depth != 1 && bit_depth != 2 &&
       bit_depth != 4 && bit_depth != 8 &&
       bit_depth != 16)
-      png_error(png_ptr, "Invalid Bit Depth Found");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid Bit Depth Found");
 
    if (color_type < 0 || color_type == 1 ||
       color_type == 5 || color_type > 6)
-      png_error(png_ptr, "Invalid Color Type Found");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid Color Type Found");
 
    if (color_type == PNG_COLOR_TYPE_PALETTE &&
       bit_depth == 16)
-      png_error(png_ptr, "Found Invalid Color Type and Bit Depth Combination");
+      (*(png_ptr->error_fn))(png_ptr, "Found Invalid Color Type and Bit Depth Combination");
 
    if ((color_type == PNG_COLOR_TYPE_RGB ||
       color_type == PNG_COLOR_TYPE_GRAY_ALPHA ||
       color_type == PNG_COLOR_TYPE_RGB_ALPHA) &&
       bit_depth < 8)
-      png_error(png_ptr, "Found Invalid Color Type and Bit Depth Combination");
+      (*(png_ptr->error_fn))(png_ptr, "Found Invalid Color Type and Bit Depth Combination");
 
    if (interlace_type > 1)
-      png_error(png_ptr, "Found Invalid Interlace Value");
+      (*(png_ptr->error_fn))(png_ptr, "Found Unsupported Interlace Value");
 
    if (compression_type > 0)
-      png_error(png_ptr, "Found Invalid Compression Value");
+      (*(png_ptr->error_fn))(png_ptr, "Found Unsupported Compression Value");
 
    if (filter_type > 0)
-      png_error(png_ptr, "Found Invalid Filter Value");
+      (*(png_ptr->error_fn))(png_ptr, "Found Unsupported Filter Value");
 
    /* set internal variables */
    png_ptr->width = width;
@@ -160,10 +160,12 @@ png_handle_PLTE(png_struct *png_ptr, png_info *info, png_uint_32 length)
    png_color *palette;
 
    if (length % 3)
-      png_error(png_ptr, "Invalid Palette Chunk");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid Palette Chunk");
 
    num = (int)length / 3;
    palette = (png_color *)png_malloc(png_ptr, num * sizeof (png_color));
+   png_ptr->do_free |= PNG_FREE_PALETTE;
+
    for (i = 0; i < num; i++)
    {
       png_byte buf[3];
@@ -189,6 +191,7 @@ png_handle_gAMA(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 4)
    {
+      (*(png_ptr->warning_fn))(png_ptr,"Incorrect gAMA chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -200,7 +203,6 @@ png_handle_gAMA(png_struct *png_ptr, png_info *info, png_uint_32 length)
       return;
 
    gamma = (float)igamma / (float)100000.0;
-
    png_read_gAMA(png_ptr, info, gamma);
    png_ptr->gamma = gamma;
 }
@@ -211,6 +213,7 @@ void
 png_handle_sBIT(png_struct *png_ptr, png_info *info, png_uint_32 length)
 {
    int slen;
+   register int index = 0;
    png_byte buf[4];
 
    if (png_ptr->color_type == PNG_COLOR_TYPE_PALETTE)
@@ -220,6 +223,7 @@ png_handle_sBIT(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != (png_uint_32)slen)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect sBIT chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -227,16 +231,28 @@ png_handle_sBIT(png_struct *png_ptr, png_info *info, png_uint_32 length)
    png_crc_read(png_ptr, buf, length);
    if (png_ptr->color_type & PNG_COLOR_MASK_COLOR)
    {
-      png_ptr->sig_bit.red = buf[0];
-      png_ptr->sig_bit.green = buf[1];
-      png_ptr->sig_bit.blue = buf[2];
-      png_ptr->sig_bit.alpha = buf[3];
+      png_ptr->sig_bit.red = buf[index++];
+      png_ptr->sig_bit.green = buf[index++];
+      png_ptr->sig_bit.blue = buf[index++];
+      png_ptr->sig_bit.gray = 0;
    }
    else
    {
+      png_ptr->sig_bit.red = 0;
+      png_ptr->sig_bit.green = 0;
+      png_ptr->sig_bit.blue = 0;
       png_ptr->sig_bit.gray = buf[0];
-      png_ptr->sig_bit.alpha = buf[1];
    }
+
+   if (png_ptr->color_type & PNG_COLOR_MASK_ALPHA)
+   {
+      png_ptr->sig_bit.alpha = buf[index++];
+   }
+   else
+   {
+      png_ptr->sig_bit.alpha = 0;
+   }
+
    png_read_sBIT(png_ptr, info, &(png_ptr->sig_bit));
 }
 #endif
@@ -251,6 +267,7 @@ png_handle_cHRM(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 32)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect cHRM chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -300,11 +317,13 @@ png_handle_tRNS(png_struct *png_ptr, png_info *info, png_uint_32 length)
    {
       if (length > png_ptr->num_palette)
       {
+         (*(png_ptr->warning_fn))(png_ptr, "Incorrect tRNS chunk length");
          png_crc_skip(png_ptr, length);
          return;
       }
 
       png_ptr->trans = png_malloc(png_ptr, length);
+      png_ptr->do_free |= PNG_FREE_TRANS;
       png_crc_read(png_ptr, png_ptr->trans, length);
       png_ptr->num_trans = (int)length;
    }
@@ -314,6 +333,7 @@ png_handle_tRNS(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
       if (length != 6)
       {
+         (*(png_ptr->warning_fn))(png_ptr, "Incorrect tRNS chunk length");
          png_crc_skip(png_ptr, length);
          return;
       }
@@ -330,6 +350,7 @@ png_handle_tRNS(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
       if (length != 2)
       {
+         (*(png_ptr->warning_fn))(png_ptr, "Incorrect tRNS chunk length");
          png_crc_skip(png_ptr, length);
          return;
       }
@@ -339,7 +360,7 @@ png_handle_tRNS(png_struct *png_ptr, png_info *info, png_uint_32 length)
       png_ptr->trans_values.gray = png_get_uint_16(buf);
    }
    else
-      png_error(png_ptr, "Invalid tRNS chunk");
+      (*(png_ptr->error_fn))(png_ptr, "Invalid tRNS chunk");
 
    png_read_tRNS(png_ptr, info, png_ptr->trans, png_ptr->num_trans,
       &(png_ptr->trans_values));
@@ -362,6 +383,7 @@ png_handle_bKGD(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != (png_uint_32)truelen)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect bKGD chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -390,12 +412,15 @@ png_handle_hIST(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 2 * png_ptr->num_palette)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect hIST chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
 
    num = (int)length / 2;
    png_ptr->hist = png_malloc(png_ptr, num * sizeof (png_uint_16));
+   png_ptr->do_free |= PNG_FREE_HIST;
+
    for (i = 0; i < num; i++)
    {
       png_byte buf[2];
@@ -417,6 +442,7 @@ png_handle_pHYs(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 9)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect pHYS chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -440,6 +466,7 @@ png_handle_oFFs(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 9)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect oFFs chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -462,6 +489,7 @@ png_handle_tIME(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    if (length != 7)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Incorrect tIME chunk length");
       png_crc_skip(png_ptr, length);
       return;
    }
@@ -484,12 +512,13 @@ png_handle_tIME(png_struct *png_ptr, png_info *info, png_uint_32 length)
 void
 png_handle_tEXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
 {
-   char *key, *text;
+   charf *key;
+   charf *text;
 
    text = NULL;
 
-   key = (char *)png_large_malloc(png_ptr, length + 1);
-   png_crc_read(png_ptr, (png_byte *)key, length);
+   key = (charf *)png_large_malloc(png_ptr, length + 1);
+   png_crc_read(png_ptr, (png_bytef *)key, length);
    key[(png_size_t)length] = '\0';
 
    for (text = key; *text; text++)
@@ -507,14 +536,15 @@ png_handle_tEXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
 void
 png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
 {
-   char *key, *text;
-   int ret;
+   charf *key;
+   charf *text;
+   int ret = Z_STREAM_END;
    png_uint_32 text_size, key_size;
 
    text = NULL;
 
    key = png_large_malloc(png_ptr, length + 1);
-   png_crc_read(png_ptr, (png_byte *)key, length);
+   png_crc_read(png_ptr, (png_bytef *)key, length);
    key[(png_size_t)length] = '\0';
 
    for (text = key; *text; text++)
@@ -523,6 +553,7 @@ png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
    /* zTXt can't have zero text */
    if (text == key + (png_size_t)length)
    {
+      (*(png_ptr->warning_fn))(png_ptr, "Zero length zTXt chunk");
       png_large_free(png_ptr, key);
       return;
    }
@@ -537,7 +568,7 @@ png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
 
    text++;
 
-   png_ptr->zstream->next_in = (png_byte *)text;
+   png_ptr->zstream->next_in = (png_bytef *)text;
    png_ptr->zstream->avail_in = (uInt)(length - (text - key));
    png_ptr->zstream->next_out = png_ptr->zbuf;
    png_ptr->zstream->avail_out = (png_size_t)png_ptr->zbuf_size;
@@ -551,6 +582,10 @@ png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
       ret = inflate(png_ptr->zstream, Z_PARTIAL_FLUSH);
       if (ret != Z_OK && ret != Z_STREAM_END)
       {
+         if (png_ptr->zstream->msg)
+            (*(png_ptr->warning_fn))(png_ptr,png_ptr->zstream->msg);
+         else
+            (*(png_ptr->warning_fn))(png_ptr,"zTXt decompression error");
          inflateReset(png_ptr->zstream);
          png_ptr->zstream->avail_in = 0;
          png_large_free(png_ptr, key);
@@ -564,23 +599,23 @@ png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
             text = png_malloc(png_ptr,
                png_ptr->zbuf_size - png_ptr->zstream->avail_out +
                   key_size + 1);
-            memcpy(text + (png_size_t)key_size, png_ptr->zbuf,
+            png_memcpy(text + (png_size_t)key_size, png_ptr->zbuf,
                (png_size_t)(png_ptr->zbuf_size - png_ptr->zstream->avail_out));
-            memcpy(text, key, (png_size_t)key_size);
+            png_memcpy(text, key, (png_size_t)key_size);
             text_size = key_size + (png_size_t)png_ptr->zbuf_size -
                png_ptr->zstream->avail_out;
             *(text + (png_size_t)text_size) = '\0';
          }
          else
          {
-            char *tmp;
+            charf *tmp;
 
             tmp = text;
             text = png_large_malloc(png_ptr, text_size +
                png_ptr->zbuf_size - png_ptr->zstream->avail_out + 1);
-            memcpy(text, tmp, (png_size_t)text_size);
+            png_memcpy(text, tmp, (png_size_t)text_size);
             png_large_free(png_ptr, tmp);
-            memcpy(text + (png_size_t)text_size, png_ptr->zbuf,
+            png_memcpy(text + (png_size_t)text_size, png_ptr->zbuf,
                (png_size_t)(png_ptr->zbuf_size - png_ptr->zstream->avail_out));
             text_size += png_ptr->zbuf_size - png_ptr->zstream->avail_out;
             *(text + (png_size_t)text_size) = '\0';
@@ -631,12 +666,12 @@ png_handle_zTXt(png_struct *png_ptr, png_info *info, png_uint_32 length)
    you want all pixels to be combined, pass 0xff (255) in mask.
 */
 void
-png_combine_row(png_struct *png_ptr, png_byte *row,
+png_combine_row(png_struct *png_ptr, png_bytef *row,
    int mask)
 {
    if (mask == 0xff)
    {
-      memcpy(row, png_ptr->row_buf + 1,
+      png_memcpy(row, png_ptr->row_buf + 1,
          (png_size_t)((png_ptr->width *
          png_ptr->row_info.pixel_depth + 7) >> 3));
    }
@@ -646,8 +681,8 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
       {
          case 1:
          {
-            png_byte *sp;
-            png_byte *dp;
+            png_bytef *sp;
+            png_bytef *dp;
             int m;
             int shift;
             png_uint_32 i;
@@ -684,8 +719,8 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
          }
          case 2:
          {
-            png_byte *sp;
-            png_byte *dp;
+            png_bytef *sp;
+            png_bytef *dp;
             int m;
             int shift;
             png_uint_32 i;
@@ -721,8 +756,8 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
          }
          case 4:
          {
-            png_byte *sp;
-            png_byte *dp;
+            png_bytef *sp;
+            png_bytef *dp;
             int m;
             int shift;
             png_uint_32 i;
@@ -758,8 +793,8 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
          }
          default:
          {
-            png_byte *sp;
-            png_byte *dp;
+            png_bytef *sp;
+            png_bytef *dp;
             png_uint_32 i;
             int pixel_bytes, m;
 
@@ -772,7 +807,7 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
             {
                if (m & mask)
                {
-                  memcpy(dp, sp, pixel_bytes);
+                  png_memcpy(dp, sp, pixel_bytes);
                }
 
                sp += pixel_bytes;
@@ -791,7 +826,7 @@ png_combine_row(png_struct *png_ptr, png_byte *row,
 
 #if defined(PNG_READ_INTERLACING_SUPPORTED)
 void
-png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
+png_do_read_interlace(png_row_info *row_info, png_bytef *row, int pass)
 {
    if (row && row_info)
    {
@@ -803,7 +838,7 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
       {
          case 1:
          {
-            png_byte *sp, *dp;
+            png_bytef *sp, *dp;
             int sshift, dshift;
             png_byte v;
             png_uint_32 i;
@@ -840,7 +875,7 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
          }
          case 2:
          {
-            png_byte *sp, *dp;
+            png_bytef *sp, *dp;
             int sshift, dshift;
             png_byte v;
             png_uint_32 i, j;
@@ -876,7 +911,7 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
          }
          case 4:
          {
-            png_byte *sp, *dp;
+            png_bytef *sp, *dp;
             int sshift, dshift;
             png_byte v;
             png_uint_32 i;
@@ -913,7 +948,7 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
          }
          default:
          {
-            png_byte *sp, *dp;
+            png_bytef *sp, *dp;
             png_byte v[8];
             png_uint_32 i;
             int j;
@@ -925,10 +960,10 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
             dp = row + (png_size_t)((final_width - 1) * pixel_bytes);
             for (i = row_info->width; i; i--)
             {
-               memcpy(v, sp, pixel_bytes);
+               png_memcpy(v, sp, pixel_bytes);
                for (j = 0; j < png_pass_inc[pass]; j++)
                {
-                  memcpy(dp, v, pixel_bytes);
+                  png_memcpy(dp, v, pixel_bytes);
                   dp -= pixel_bytes;
                }
                sp -= pixel_bytes;
@@ -944,8 +979,8 @@ png_do_read_interlace(png_row_info *row_info, png_byte *row, int pass)
 #endif
 
 void
-png_read_filter_row(png_row_info *row_info, png_byte *row,
-   png_byte *prev_row, int filter)
+png_read_filter_row(png_row_info *row_info, png_bytef *row,
+   png_bytef *prev_row, int filter)
 {
    switch (filter)
    {
@@ -955,8 +990,8 @@ png_read_filter_row(png_row_info *row_info, png_byte *row,
       {
          png_uint_32 i;
          int bpp;
-         png_byte *rp;
-         png_byte *lp;
+         png_bytef *rp;
+         png_bytef *lp;
 
          bpp = (row_info->pixel_depth + 7) / 8;
          for (i = (png_uint_32)bpp, rp = row + bpp, lp = row;
@@ -969,8 +1004,8 @@ png_read_filter_row(png_row_info *row_info, png_byte *row,
       case 2:
       {
          png_uint_32 i;
-         png_byte *rp;
-         png_byte *pp;
+         png_bytef *rp;
+         png_bytef *pp;
 
          for (i = 0, rp = row, pp = prev_row;
             i < row_info->rowbytes; i++, rp++, pp++)
@@ -983,9 +1018,9 @@ png_read_filter_row(png_row_info *row_info, png_byte *row,
       {
          png_uint_32 i;
          int bpp;
-         png_byte *rp;
-         png_byte *pp;
-         png_byte *lp;
+         png_bytef *rp;
+         png_bytef *pp;
+         png_bytef *lp;
 
          bpp = (row_info->pixel_depth + 7) / 8;
          for (i = 0, rp = row, pp = prev_row;
@@ -1005,10 +1040,10 @@ png_read_filter_row(png_row_info *row_info, png_byte *row,
       {
          int bpp;
          png_uint_32 i;
-         png_byte *rp;
-         png_byte *pp;
-         png_byte *lp;
-         png_byte *cp;
+         png_bytef *rp;
+         png_bytef *pp;
+         png_bytef *lp;
+         png_bytef *cp;
 
          bpp = (row_info->pixel_depth + 7) / 8;
          for (i = 0, rp = row, pp = prev_row,
@@ -1058,7 +1093,7 @@ png_read_finish_row(png_struct *png_ptr)
    if (png_ptr->interlaced)
    {
       png_ptr->row_number = 0;
-      memset(png_ptr->prev_row, 0, (png_size_t)png_ptr->rowbytes + 1);
+      png_memset(png_ptr->prev_row, 0, (png_size_t)png_ptr->rowbytes + 1);
       do
       {
          png_ptr->pass++;
@@ -1103,19 +1138,19 @@ png_read_finish_row(png_struct *png_ptr)
                png_byte buf[4];
                png_uint_32 crc;
 
-               png_read_data(png_ptr, buf, 4);
+               (*(png_ptr->read_data_fn))(png_ptr, buf, 4);
                crc = png_get_uint_32(buf);
                if (((crc ^ 0xffffffffL) & 0xffffffffL) !=
                   (png_ptr->crc & 0xffffffffL))
-                  png_error(png_ptr, "Bad CRC value");
+                  (*(png_ptr->error_fn))(png_ptr, "Bad CRC value");
 
-               png_read_data(png_ptr, buf, 4);
+               (*(png_ptr->read_data_fn))(png_ptr, buf, 4);
                png_ptr->idat_size = png_get_uint_32(buf);
                png_reset_crc(png_ptr);
 
                png_crc_read(png_ptr, buf, 4);
                if (memcmp(buf, png_IDAT, 4))
-                  png_error(png_ptr, "Not enough image data");
+                  (*(png_ptr->error_fn))(png_ptr, "Not enough image data");
 
             }
             png_ptr->zstream->avail_in = (uInt)png_ptr->zbuf_size;
@@ -1130,22 +1165,22 @@ png_read_finish_row(png_struct *png_ptr)
          {
             if (!(png_ptr->zstream->avail_out) || png_ptr->zstream->avail_in ||
                png_ptr->idat_size)
-               png_error(png_ptr, "Extra compressed data");
+               (*(png_ptr->warning_fn))(png_ptr, "Extra compressed data");
             png_ptr->mode = PNG_AT_LAST_IDAT;
             break;
          }
          if (ret != Z_OK)
-            png_error(png_ptr, "Compression Error");
+            (*(png_ptr->error_fn))(png_ptr, "Compression Error");
 
          if (!(png_ptr->zstream->avail_out))
-            png_error(png_ptr, "Extra compressed data");
+            (*(png_ptr->error_fn))(png_ptr, "Extra compressed data");
 
       } while (1);
       png_ptr->zstream->avail_out = 0;
    }
 
    if (png_ptr->idat_size || png_ptr->zstream->avail_in)
-      png_error(png_ptr, "Extra compression data");
+      (*(png_ptr->warning_fn))(png_ptr, "Extra compression data");
 
    inflateReset(png_ptr->zstream);
 
@@ -1257,18 +1292,18 @@ png_read_start_row(png_struct *png_ptr)
       1 + ((max_pixel_depth + 7) >> 3);
 #ifdef PNG_MAX_MALLOC_64K
    if (rowbytes > 65536L)
-      png_error(png_ptr, "This image requires a row greater then 64KB");
+      (*(png_ptr->error_fn))(png_ptr, "This image requires a row greater then 64KB");
 #endif
-   png_ptr->row_buf = (png_byte *)png_large_malloc(png_ptr, rowbytes);
+   png_ptr->row_buf = (png_bytef *)png_large_malloc(png_ptr, rowbytes);
 
 #ifdef PNG_MAX_MALLOC_64K
    if (png_ptr->rowbytes + 1 > 65536L)
-      png_error(png_ptr, "This image requires a row greater then 64KB");
+      (*(png_ptr->error_fn))(png_ptr, "This image requires a row greater then 64KB");
 #endif
    png_ptr->prev_row = png_large_malloc(png_ptr,
       png_ptr->rowbytes + 1);
 
-   memset(png_ptr->prev_row, 0, (png_size_t)png_ptr->rowbytes + 1);
+   png_memset(png_ptr->prev_row, 0, (png_size_t)png_ptr->rowbytes + 1);
 
    png_ptr->row_init = 1;
 }
